#!/usr/local/bin/scsh \
-e main -s
!#

;;; make-scsh-image.scm --- create a scsh image

;;; Copyright (C) 2001 by L.P. data systems

;;; Author: Walter C. Pelissero <walter@pelissero.org>
;;; $Id: make-scsh-image.scm,v 1.3 2003/11/20 15:48:01 wcp Exp $

;;; This program is free software; you can redistribute it and/or
;;; modify it under the terms of the GNU General Public License as
;;; published by the Free Software Foundation; either version 2, or
;;; (at your option) any later version.
;;; This program is distributed in the hope that it will be useful,
;;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
;;; General Public License for more details.
;;; You should have received a copy of the GNU General Public License
;;; along with this program; see the file COPYING.  If not, write to
;;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;;; Boston, MA 02111-1307, USA.

;;; Commentary:

(define (main args)
  (if (< (length args) 2)
      (format #t "usage: ~a source_file [args]\n" (car args))
      (let* ((source-file (cadr args))
	     (out-file (replace-extension source-file ".image"))
	     (args (cddr args))
	     (temp-file #f))
	(dynamic-wind
	 (lambda ()
	   (set! temp-file (create-temp-file)))
	 (lambda ()
	   (if (zero? (run (| (begin
				(format #t "(dump-scsh-program main \"~a\") (exit)" temp-file))
			      (scsh ,@args -lm ,source-file -o program))
			   (> /dev/null)))
	       (begin
		 (substitue-first-line temp-file out-file)
		 (set-file-mode out-file #o755))))
	 (lambda ()
	   (delete-file temp-file))))))

(define (substitue-first-line in out)
  (call-with-input-file in
    (lambda (in)
      (call-with-output-file out
	(lambda (out)
	  ;; skip the first line
	  (read-line in)
	  (display "#!/usr/local/lib/scsh/scshvm \\\n-o /usr/local/lib/scsh/scshvm -i\n!#\n" out)
	  (let ((buffer (make-string 10240)))
	    (let loop ((bytes (read-string! buffer in)))
	      (if bytes
		  (begin
		    (write-string buffer out 0 bytes)
		    (loop (read-string! buffer in)))))))))))

;;;
;;; Leave the following lines at end of file.
;;;
;;; Local Variables:
;;; mode:scheme
;;; scheme-program-name:"scsh"
;;; End:
